@@ -2,15 +2,15 @@
 #' @param model object type with an available `tidy` method.
 #' @inheritParams modelsummary
 #' @return data.frame with side-by-side model summaries
-#' @keywords internal
+#' @noRd
 extract_estimates <- function(
   model,
-  estimate = "estimate",
-  statistic = "std.error",
-  statistic_override = NULL,
+  estimate   = "estimate",
+  statistic  = "std.error",
+  vcov       = NULL,
   conf_level = .95,
-  fmt = "%.3f",
-  stars = FALSE,
+  fmt        = "%.3f",
+  stars      = FALSE,
   ...) {
 
   # conf.int to glue
@@ -31,8 +31,9 @@ extract_estimates <- function(
     sprintf("{%s}", estimate_glue))
     
   # statistics to glue
-  if (!is.null(statistic_override) &&     # don't add parentheses to manual strings
-      is.character(statistic_override)) {
+  if (!is.null(vcov) &&     # don't add parentheses
+      is.character(vcov) && # to manual strings
+      length(vcov) > 1) {   # of length greater than 1 (i.e., robust shortcuts)
     statistic_glue <- ifelse(
       grepl("\\{", statistic_glue),
       statistic_glue,
@@ -49,31 +50,33 @@ extract_estimates <- function(
   
   # extract estimates using broom or parameters
   if (any(grepl("conf", estimate_glue))) {
-    est <- get_estimates(model, conf_level=conf_level)
+    est <- get_estimates(model, conf_level=conf_level, ...)
   } else {
-    est <- get_estimates(model, conf_level=NULL)
+    est <- get_estimates(model, conf_level=NULL, ...)
   }
   
   # estimate override
-  if (!is.null(statistic_override)) {
+  flag1 <- !is.null(vcov)
+  flag2 <- isFALSE(all.equal(vcov, stats::vcov))
+  flag3 <- !is.character(vcov)
+  flag4 <- is.character(vcov) && length(vcov) == 1 && !vcov %in% c("classical", "iid", "constant")
+  flag5 <- is.character(vcov) && length(vcov) > 1
+
+  if (flag1 && (flag2 || flag3 || flag4 || flag5)) {
 
     # extract overriden estimates
-    so <- extract_statistic_override(
+    so <- extract_vcov(
       model,
-      statistic_override=statistic_override,
+      vcov=vcov,
       conf_level=conf_level)
 
-    # keep only columns that do not appear in so
-    est <- est[, c('term', base::setdiff(colnames(est), colnames(so))), drop = FALSE]
-
-    # make sure statistic_override is of the correct length
-    if (nrow(est) != nrow(so)) {
-      stop("statistic_override and estimates have different dimensions.")
-    }
-
-    # merge statistic_override and estimates
-    est <- merge(est, so, by="term", sort=FALSE)
+    if (!is.null(so) && nrow(est) == nrow(so)) {
+      # keep only columns that do not appear in so
+      est <- est[, c('term', base::setdiff(colnames(est), colnames(so))), drop = FALSE]
+      # merge vcov and estimates
+      est <- merge(est, so, by="term", sort=FALSE)
 
+    } 
   }
 
   # tidy_custom
@@ -101,23 +104,39 @@ extract_estimates <- function(
     }
   }
 
+
   # term must be a character (not rounded with decimals when integer)
   est$term <- as.character(est$term)
 
-  # round everything
-  for (n in colnames(est)) {
-    est[[n]] <- rounding(est[[n]], fmt)
-  }
 
-  # stars
-  if (!isFALSE(stars)) {
+  # stars (before rounding)
+  # compute stars if: 
+  #   a) stars is not FALSE
+  #   b) {stars} is in glue string 
+  # paste stars if:
+  #   a) stars is not FALSE and is not a glue string
+  is_star <- !isFALSE(stars) || isTRUE(any(grepl("\\{stars\\}", estimate_glue)))
+  is_glue <- grepl("\\{", estimate)
+  if (is_star) {
+    # glue string can include stars even if stars=FALSE 
+    if (isFALSE(stars)) {
+      stars <- TRUE
+    }
     if (!'p.value' %in% colnames(est)) {
       stop('To use the `stars` argument, the `tidy` function must produce a column called "p.value"')
     }
     est$stars <- make_stars(est$p.value, stars)
+  }
+  if (isTRUE(is_star) && isFALSE(is_glue)) {
     estimate_glue[1] <- paste0(estimate_glue[1], "{stars}")
   }
 
+
+  # round everything
+  for (n in colnames(est)) {
+    est[[n]] <- rounding(est[[n]], fmt)
+  }
+
   # extract estimates (there can be several)
   for (i in seq_along(estimate_glue)) {
     s <- estimate_glue[i]
@@ -126,6 +145,7 @@ extract_estimates <- function(
     est[[paste0("modelsummary_tmp", i)]][est[[s]] == ""] <- ""
   }
 
+
   # subset columns
   cols <- c('term', paste0('modelsummary_tmp', seq_along(estimate_glue)))
   est <- est[, cols, drop = FALSE]
@@ -157,7 +177,13 @@ extract_estimates <- function(
   return(est)
 }
 
-get_estimates <- function(model, conf_level=NULL, ...) {
+#' Extract model estimates. A mostly internal function with some potential uses
+#' outside.
+#'
+#' @inheritParams modelsummary
+#' @param model a single model object
+#' @export
+get_estimates <- function(model, conf_level=.95, ...) {
 
   if (is.null(conf_level)) {
     conf_int=FALSE
@@ -171,31 +197,36 @@ get_estimates <- function(model, conf_level=NULL, ...) {
     "term" %in% colnames(x)
   }
 
-  # tidy method in the environment takes precedence over everything
+  # generics first
   est <- suppressWarnings(try(
     generics::tidy(model, conf.int=conf_int, conf.level=conf_level, ...), silent=TRUE))
   if (flag(est)) return(est)
 
-  # easystats/parameters: unless broom is loaded manually, this is the default method
+  # broom second 
+  # can't use generics::tidy here otherwise broom only gets loaded the second
+  # time modelsummary is called, and the first attempt fails.
   est <- suppressWarnings(try(
-    tidy_easystats(model, ci=conf_level, ...), silent=TRUE))
+    broom::tidy(model, conf.int=conf_int, conf.level=conf_level, ...), silent=TRUE))
   if (flag(est)) return(est)
 
-  # if broom is installed locally, try it
-  if (check_dependency("broom")) {
-    est <- suppressWarnings(try(
-      broom::tidy(model, conf.int=conf_int, conf.level=conf_level, ...), silent=TRUE))
+  # parameters third
+  f <- tidy_easystats <- function(model, ...) {
+    msg <- utils::capture.output(out <- parameters::model_parameters(model, ...))
+    out <- parameters::standardize_names(out, style="broom")
+    return(out)
   }
+  est <- suppressWarnings(try(
+    f(model, ci=conf_level, ...), silent=TRUE))
   if (flag(est)) return(est)
 
-  # if broom.mixed is installed locally, try it
+  # broom.mixed fourth
   if (check_dependency("broom.mixed")) {
     est <- suppressWarnings(try(
       broom.mixed::tidy(model, conf.int=TRUE, conf.level=conf_level, ...), silent=TRUE))
   }
   if (flag(est)) return(est)
 
-  stop(sprintf('Cannot extract the required information from models of class "%s". Consider installing and loading `broom.mixed`, or any other package that includes `tidy` and `glance` methods appropriate for this model type. Alternatively, you can easily define your own methods by following the instructions on the `modelsummary` website: https://vincentarelbundock.github.io/modelsummary/articles/newmodels.html', class(model)[1]))
+  stop(sprintf('Cannot extract the required information from models of class "%s". Consider installing `broom.mixed`, or any other package that includes `tidy` and `glance` methods appropriate for this model type. Alternatively, you can easily define your own methods by following the instructions on the `modelsummary` website: https://vincentarelbundock.github.io/modelsummary/articles/newmodels.html', class(model)[1]))
 
 }
  
