@@ -1,6 +1,5 @@
-context('coef_map')
-
-library(modelsummary)
+# CRAN win-devel does not play well with fancy unicode x
+skip_on_cran()
 
 test_that(": in interactions become x", {
   mod <- lm(am ~ drat * mpg * vs, mtcars)
